mult_testing_par <- function(muvec,b,corr=FALSE,autocorr=FALSE,omitted_var=FALSE,full_optim=FALSE,dist="norm"){
  #DESCRIPTION
  # Runs parallel simulations and calculations on instances of the same data generating process
  # DGP is always given by y = P(x*b>0), with x random
  #-------------------------------------------------------------------------------
  #ARGUMENTS
  # muvec:        preferences for which output needs to be generated
  # b:            linear multiplier on RHS-variables x
  # corr:         if TRUE, simulate x correlated with correlation of 0.5
  # autocorr:     if TRUE, simulate x as autocorrelated process with lag coefficients rho_1 = 0.7 and rho_2 = -0.1
  # omitted_var:  if TRUE, drop x_1 from simulated explanatory variables
  # full_optim:   if TRUE, use all available optimization algorithms for weighted methods. Instead, use only ucminf (in weighttesting_thresh)
  # dist:         Distribution choice for data x (normal, cauchy, exponential)
  #-------------------------------------------------------------------------------
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  library(foreach)
  library(doParallel)
  library(plyr)
  library(optimx)
  cl<-makeCluster(32)
  registerDoParallel(cl)
  functionlist <- c("randomdata","weighttesting_thresh","LL_wlogit","LL_wprobit","gr_wlogit","gr_wprobit","findThreshold")
  
  set.seed(111)
  
  Tsteps <- round(10^seq(2,4,0.1))
  runs <- 100
  runs_max <- 50000
  
  res_byrun <- function(r){
    if (r%%100==0) {print(paste("run",r))}
    data_obj <- randomdata(T=T,b=b,corr=corr,autocorr=autocorr,dist=dist)
    data_obj$subset <- c(1:ceiling(T/2))
    if (omitted_var) {data_obj$model <- data_obj$model[2:length(data_obj$model)]}
    table <- weighttesting_thresh(data_obj,mu=muvec,long=FALSE,oos=TRUE,full_optim=full_optim)
    dsize <- dim(table)
    dnames <- dimnames(table)  

    dim(table) <- c(dsize[1]*dsize[2],dsize[3])
    colnames(table) <- dnames$res
    table <- data.frame(table)
    table <- cbind(r=r,mu=muvec,method=rep(dnames$method,each=length(dnames$mu)),table)
    return(table)
  }
  
  check_runs <- function(p,beta=0.005,alpha_vec=c(0.01,0.05,0.1,0.5),runs_end){
    #depending on the value of p, this runs up to 2 tests: rejection of p versus its adjacent values in alpha_vec
    if (p==0 | p==1){return(TRUE)}
    if (length(which(p==alpha_vec))==1){return(FALSE)}
    
    alpha_vec1 <- c(sort(alpha_vec),1)
    alpha_vec2 <- c(0,sort(alpha_vec,decreasing=TRUE))
    testalpha1 <- alpha_vec1[which(alpha_vec1>p)[1]]    #p smaller than alpha
    testalpha2 <- alpha_vec2[which(alpha_vec2<p)[1]]    #p larger than alpha
    if (testalpha1==1){check1 <- TRUE}else{check1 <- runs_end*p < qnorm(beta,testalpha1*runs_end,sqrt(testalpha1*(1-testalpha1)*runs_end))}
    if (testalpha2==0){check2 <- TRUE}else{check2 <- runs_end*p > qnorm(1-beta,testalpha2*runs_end,sqrt(testalpha2*(1-testalpha2)*runs_end))}
    return(check1 & check2)
  }
  
  n_mu <- length(muvec)
  n_methods <- 6
  n_res <- 18
  res_table <- mat.or.vec(n_mu*length(Tsteps)*n_methods,n_res)
  colnames(res_table) <- c("T","runs","mu","method","lambda_m","lambda_sd","lambda_05","lambda_95","Ua_in_m","p_Ua_in","Ur_in_m","p_Ur_in",
                           "Ua_out_m","p_Ua_out","Ur_out_m","p_Ur_out","R2","R2adj")
  res_table <- data.frame(res_table)
  pos <- 1
#   for (mu in muvec){
#     print(paste("setting mu to",mu))
  for (T in Tsteps){
    print(paste("starting simulations for",T,"periods"))
    check <- FALSE
    long_res <- NULL
    runs_start <- 1
    runs_end <- runs
    while (!check & runs_end<=runs_max){
      temp <- foreach (r=runs_start:runs_end,.combine=rbind,.export=functionlist) %dopar% (res_byrun(r))
      long_res <- rbind(long_res,data.frame(temp))
      res_dd <- ddply(long_res,.(mu,method),summarize,
                      lambda_m=mean(lambda),lambda_sd=sd(lambda),
                      lambda_05=quantile(lambda,0.05),lambda_95=quantile(lambda,0.95),
                      Ua_in=mean(Ua_in),p_Ua_in=mean(diffa_in>0),
                      Ur_in_m=mean(Ur_in),p_Ur_in=mean(diffr_in>0),
                      Ua_out_m=mean(Ua_out),p_Ua_out=mean(diffa_out>0),
                      Ur_out_m=mean(Ur_out),p_Ur_out=mean(diffr_out>0),
                      R2=mean(R2),R2adj=mean(R2adj))
      res_dd[,2] <- as.character(res_dd[,2])
      
      p <- c(as.matrix(res_dd[1:10,grep("p_U",colnames(res_dd))]))
      check <- TRUE
      if(runs_end<runs_max){
        for (k in 1:length(p)){
          check <- check & check_runs(p[k],runs_end=runs_end)
          if (check == FALSE){
            runs_start <- runs_end+1
            runs_end <- min(2*runs_end,runs_max)
            print(paste("Unclear p-values; runs_end extended to",runs_end))
            rm(res_dd)
            break
          }
        }
      }
    }
    
    
    # save results
    
    
    res_table[pos:(pos+n_methods*n_mu-1),"T"] <- T
    res_table[pos:(pos+n_methods*n_mu-1),"runs"] <- runs_end
    res_table[pos:(pos+n_methods*n_mu-1),3:n_res] <- res_dd
    pos <- pos+n_methods*n_mu
    rm(temp,long_res)
  }

  stopCluster(cl)
  return(res_table)
}