weighttesting_thresh <- function(data_obj,muvec,oos=FALSE,long=FALSE,full_optim=FALSE,b_U_Sarlin = TRUE){
  #DESCRIPTION
  # Performs different estimations to compare threshold setting to weighted probit and logit estimations
  #-------------------------------------------------------------------------------
  #USAGE
  # weighttesting(data,LHS,model,mu,long=FALSE,oos=FALSE)
  #-------------------------------------------------------------------------------
  #ARGUMENTS
  # data:         data frame or matrix containing the data
  # LHS:          string indicating the LHS variable
  # model:        vector of strings indicator RHS variables (including intercept!)
  # mu:           policy makers preference for false alarms (mu>0.5) or missed crises (mu<0.5)
  # long:         boolean to indicate that additional output should be returned
  # oos:          boolean to indicate whether there should also be out-of-sample comparisons (on 50% of the sample)
  #-------------------------------------------------------------------------------
  # Gregor von Schweinitz and Peter Sarlin, 2018-07-25
  
  library("optimx")
  
  eps <- 10^-10
  
  methods <- c("Probit opt threshold","Probit set threshold","Weighted probit",
               "Logit opt threshold","Logit set threshold","Weighted logit")
  
  if (oos){
    resnames <- c("lambda","A_in","B_in","C_in","D_in","L_in","Ua_in","Ur_in","A_out","B_out","C_out","D_out","L_out","Ua_out","Ur_out","diffa_in","diffr_in","diffa_out","diffr_out","R2","R2adj")
    cols_in <- c(1:8)
    cols_out <- c(2:8)
  }
  else{
    resnames <- c("lambda","A","B","C","D","L","Ua","Ur","R2","R2adj")
  }
  res_compare <- array(dim=c(length(muvec),length(methods),length(resnames)),
                       dimnames=list(mu=muvec,method=methods,res=resnames))
  
  
  data <- data_obj$data
  model <- data_obj$model
  LHS <- data_obj$LHS
  subset <- data_obj$subset
  N <- length(subset)
  pformula <- paste(LHS,"~",paste(model,collapse=" + "),"-1")
  pformula_const <- paste(LHS,"~Intercept-1")
  Y_est <- c(data_obj$data[subset,LHS])
  X_est <- as.matrix(data_obj$data[subset,model])
  if (oos){
    Y_oos <- c(data_obj$data[-subset,LHS])
    X_oos <- as.matrix(data_obj$data[-subset,model])
  }
  P1 <- mean(Y_est,na.rm=TRUE)
  P2 <- 1-P1
  #Transforming additive threshold weights into multiplicative LL weights with mean 1
  # tau is the true share of events that is needed such that tau/P2 : tau/P1 = mu : (1-mu)
  if (b_U_Sarlin){
    tauvec <- (muvec*P1)/(muvec*P1+(1-muvec)*P2)
    threshvec <- 1-muvec
  }else{
    tauvec <- muvec
    threshvec <- (1-muvec)*P1 / ((1-muvec)*P1+muvec*P2)
  }
  
  
  # -------------------------------------
  # standard probit case
  res_probit <- glm(formula=pformula,data=data,family=binomial(link="probit"),model=FALSE,subset=subset)
  res_const <- glm(formula=pformula_const,data=data,family=binomial(link="probit"),model=FALSE,subset=subset)
  # getting data to work with
  R2 <- 1-res_probit$deviance/res_const$deviance
  R2adj <- R2 - 2*(length(model)-1)/res_const$deviance
  
  table <- findThreshold(Y_est,res_probit$fitted.values,muvec,lambda_set=sort(unique(res_probit$fitted.values)),b_U_Sarlin = b_U_Sarlin)
  if (oos){
    p_oos <- pnorm(X_oos%*%res_probit$coefficients)
    diff <- rep(0,4)
    for (m in 1:length(muvec)){
      lambda <- table[m,"lambda"]
#       table_oos <- findThreshold(Y_oos,p_oos,muvec[m],lambda,P1,P2,b_U_Sarlin = b_U_Sarlin)
      table_oos <- findThreshold(Y_oos,p_oos,muvec[m],lambda,b_U_Sarlin = b_U_Sarlin)
      res_compare[m,1,] <- c(as.matrix(table[m,cols_in]),as.matrix(table_oos[1,cols_out]),diff,R2,R2adj)
    }
  }
  else{
    res_compare[,1,] <- cbind(table,R2=R2,R2adj=R2adj)
  }
  #probit with set threshold
  for (m in 1:length(muvec)){
    mu <- muvec[m]
    thresh <-  threshvec[m]
    table <- findThreshold(Y_est,res_probit$fitted.values,mu,lambda_set=thresh,b_U_Sarlin = b_U_Sarlin)
    if (oos){
#       table_oos <- findThreshold(Y_oos,p_oos,mu,lambda_set=thresh,P1,P2,b_U_Sarlin = b_U_Sarlin)
      table_oos <- findThreshold(Y_oos,p_oos,mu,lambda_set=thresh,b_U_Sarlin = b_U_Sarlin)
      diff <- c(table[1,c("Ua","Ur")]-res_compare[m,1,c("Ua_in","Ur_in")],
                table_oos[1,c("Ua","Ur")]-res_compare[m,1,c("Ua_out","Ur_out")])
      res_compare[m,2,] <- c(as.matrix(table[1,cols_in]),as.matrix(table_oos[1,cols_out]),diff,R2,R2adj)
    }
    else{
      res_compare[m,2,] <- c(as.matrix(table[1,]),R2,R2adj)
    }
  }
  
  # -------------------------------------
  # weighted probit case
#   print("wprobit")
  for (m in 1:length(muvec)){
    mu <- muvec[m]
    tau <- tauvec[m]
    w <- Y_est
    w[Y_est==1] <- tau/P1
    w[Y_est==0] <- (1-tau)/P2
    
    LL <- function(x){LL_wprobit(Y=Y_est,X=X_est,coefficients=x,w=w,long=FALSE)}
    gr <- function(x){gr_wprobit(Y=Y_est,X=X_est,coefficients=x,w=w)}
    hess <- function(x){hess_wprobit(Y=Y_est,X=X_est,coefficients=x,w=w)}
    
    coeff_const <- c(rep(0,length(model)-1),qnorm(mean(w*Y_est)))
    startvalues <- c(res_probit$coefficients)
    if (!is.finite(LL(startvalues))){startvalues <- rep(0,length(startvalues))}
    res_opt <- tryCatch(
      if (full_optim){optimx(par=startvalues,fn=LL,gr=gr,control=list(all.methods=TRUE))}
      else{optimx(par=startvalues,fn=LL,gr=gr,method="ucminf")},
      error = function(err){
        print("error encountered - trying again without gradient")
        if (full_optim){res <- optimx(par=startvalues,fn=LL,control=list(all.methods=TRUE))}
        else{res <- optimx(par=startvalues,fn=LL,method="ucminf")}
        return(res)
      }
    )
    if (full_optim){optrow <- which(res_opt[,value]==min(res_opt[,value]))}else{optrow=1}
    res_wprobit <- LL_wprobit(Y=Y_est,X=X_est,coefficients=c(as.matrix(res_opt[optrow,1:length(startvalues)])),w=w,long=TRUE)
    res_wprobit$LL_null <- -LL_wprobit(Y=Y_est,X=X_est,coefficients=coeff_const,w=w,long=FALSE)
  
    R2 <- 1-res_wprobit$LL/res_wprobit$LL_null
    R2adj <- R2 + (length(model)-1)/res_wprobit$LL_null
    if (long){
      names(res_wprobit$coefficients) <- model
      res_wprobit$hess <- hess(res_wprobit$coefficients)
      res_wprobit$se <- sqrt(abs(diag(qr.solve(res_wprobit$hess,tol=eps))))
      df <- N - length(model)
      res_wprobit$p <- 1-2*abs(0.5-pt(as.matrix(res_wprobit$coefficients/res_wprobit$se),df))
    }
    
    table <- findThreshold(Y_est,res_wprobit$fitted.values,mu,lambda_set=0.5,b_U_Sarlin = b_U_Sarlin)
    if (oos){
      p_oos <- pnorm(X_oos%*%res_wprobit$coefficients)
#       table_oos <- findThreshold(Y_oos,p_oos,mu,lambda_set=0.5,P1,P2,b_U_Sarlin = b_U_Sarlin)
      table_oos <- findThreshold(Y_oos,p_oos,mu,lambda_set=0.5,b_U_Sarlin = b_U_Sarlin)
      diff <- c(table[1,c("Ua","Ur")]-res_compare[m,1,c("Ua_in","Ur_in")],
                table_oos[1,c("Ua","Ur")]-res_compare[m,1,c("Ua_out","Ur_out")])
      res_compare[m,3,] <- c(as.matrix(table[1,cols_in]),as.matrix(table_oos[,cols_out]),diff,R2=R2,R2adj=R2adj)
    }
    else{
      res_compare[m,3,] <- c(as.matrix(table[1,]),R2=R2,R2adj=R2adj)
    }
  }
  
  
  # -------------------------------------
  # standard logit case
#   print("logit")
  pformula <- paste(LHS,"~",paste(model,collapse=" + "),"-1")
  res_logit <- glm(formula=pformula,data=data,family=binomial(link="logit"),model=FALSE,subset=subset)
  res_const <- glm(formula=pformula_const,data=data,family=binomial(link="logit"),model=FALSE,subset=subset)
  R2 <- 1-res_logit$deviance/res_const$deviance
  R2adj <- R2 - 2*(length(model)-1)/res_const$deviance
  table <- findThreshold(Y_est,res_logit$fitted.values,muvec,lambda_set=sort(unique(res_logit$fitted.values)),b_U_Sarlin = b_U_Sarlin)
  if (oos){
    p_oos <- plogis(X_oos%*%res_logit$coefficients)
    diff <- rep(0,4)
    for (m in 1:length(muvec)){
      lambda <- table[m,"lambda"]
#       table_oos <- findThreshold(Y_oos,p_oos,muvec[m],lambda,P1,P2,b_U_Sarlin = b_U_Sarlin)
      table_oos <- findThreshold(Y_oos,p_oos,muvec[m],lambda,b_U_Sarlin = b_U_Sarlin)
      res_compare[m,4,] <- c(as.matrix(table[m,cols_in]),as.matrix(table_oos[1,cols_out]),diff,R2,R2adj)
    }
  }
  else{
    res_compare[,4,] <- cbind(table,R2=R2,R2adj=R2adj)
  }
  
  #logit with set threshold
  for (m in 1:length(muvec)){
    mu <- muvec[m]
    thresh <-  threshvec[m]
    table <- findThreshold(Y_est,res_logit$fitted.values,mu,lambda_set=thresh,b_U_Sarlin = b_U_Sarlin)
    if (oos){
#       table_oos <- findThreshold(Y_oos,p_oos,mu,lambda_set=thresh,P1,P2,b_U_Sarlin = b_U_Sarlin)
      table_oos <- findThreshold(Y_oos,p_oos,mu,lambda_set=thresh,b_U_Sarlin = b_U_Sarlin)
      diff <- c(table[1,c("Ua","Ur")]-res_compare[m,4,c("Ua_in","Ur_in")],
                table_oos[1,c("Ua","Ur")]-res_compare[m,4,c("Ua_out","Ur_out")])
      res_compare[m,5,] <- c(as.matrix(table[1,cols_in]),as.matrix(table_oos[1,cols_out]),diff,R2,R2adj)
    }
    else{
      res_compare[m,5,] <- c(as.matrix(table[1,]),R2,R2adj)
    }
  }

  # -------------------------------------
  # weighted logit case
#   print("wlogit")
  for (m in 1:length(muvec)){
    mu <- muvec[m]
    tau <- tauvec[m]
    w <- Y_est
    w[Y_est==1] <- tau/P1
    w[Y_est==0] <- (1-tau)/P2
    
    LL <- function(x){LL_wlogit(Y=Y_est,X=X_est,coefficients=x,w=w,long=FALSE)}
    gr <- function(x){gr_wlogit(Y=Y_est,X=X_est,coefficients=x,w=w)}
    hess <- function(x){hess_wlogit(Y=Y_est,X=X_est,coefficients=x,w=w)}
    
    coeff_const <- c(rep(0,length(model)-1),qlogis(mean(w*Y_est)))
    startvalues <- c(res_logit$coefficients)
    if (!is.finite(LL(startvalues))){startvalues <- rep(0,length(startvalues))}
    res_opt <- tryCatch(
      if (full_optim){optimx(par=startvalues,fn=LL,gr=gr,control=list(all.methods=TRUE))}
      else{optimx(par=startvalues,fn=LL,gr=gr,method="ucminf")},
      error = function(err){
        print("error encountered - trying again without gradient")
        if (full_optim){res <- optimx(par=startvalues,fn=LL,control=list(all.methods=TRUE))}
        else{res <- optimx(par=startvalues,fn=LL,method="ucminf")}
        return(res)
      }
    )
    if (full_optim){optrow <- which(res_opt[,value]==min(res_opt[,value]))}else{optrow=1}
    res_wlogit <- LL_wlogit(Y=Y_est,X=X_est,coefficients=c(as.matrix(res_opt[optrow,1:length(startvalues)])),w=w,long=TRUE)
    res_wlogit$LL_null <- -LL_wlogit(Y=Y_est,X=X_est,coefficients=coeff_const,w=w,long=FALSE)
  
    R2 <- 1-res_wlogit$LL/res_wlogit$LL_null
    R2adj <- R2 + (length(model)-1)/res_wlogit$LL_null
    if (long){
      names(res_wlogit$coefficients) <- model
      res_wlogit$hess <- hess(res_wlogit$coefficients)
      res_wlogit$se <- sqrt(abs(diag(qr.solve(res_wlogit$hess,tol=eps))))
      df <- N - length(model)
      res_wlogit$p <- 1-2*abs(0.5-pt(as.matrix(res_wlogit$coefficients/res_wlogit$se),df))
    }
    
    table <- findThreshold(Y_est,res_wlogit$fitted.values,mu,lambda_set=0.5,b_U_Sarlin = b_U_Sarlin)
    if (oos){
      p_oos <- plogis(X_oos%*%res_wlogit$coefficients)
#       table_oos <- findThreshold(Y_oos,p_oos,mu,lambda_set=0.5,P1,P2,b_U_Sarlin = b_U_Sarlin)
      table_oos <- findThreshold(Y_oos,p_oos,mu,lambda_set=0.5,b_U_Sarlin = b_U_Sarlin)
      diff <- c(table[1,c("Ua","Ur")]-res_compare[m,4,c("Ua_in","Ur_in")],
                table_oos[1,c("Ua","Ur")]-res_compare[m,4,c("Ua_out","Ur_out")])
      res_compare[m,6,] <- c(as.matrix(table[1,cols_in]),as.matrix(table_oos[1,cols_out]),diff,R2=R2,R2adj=R2adj)
    }
    else{
      res_compare[m,6,] <- c(as.matrix(table[1,]),R2=R2,R2adj=R2adj)
    }
  }
  
  

  if (long){
    out <- NULL
    out$res_compare <- res_compare
    out$probit <- res_probit
    out$wprobit <- res_wprobit
    out$logit <- res_logit
    out$wlogit <- res_wlogit
    return(out)
  }
  else{return(res_compare)}
}